@@ -58,7 +58,7 @@ struct rgbvec {
 
 typedef struct LUT3DContext {
     const AVClass *class;
-    enum interp_mode interpolation;
+    int interpolation;          ///<interp_mode
     char *file;
     uint8_t rgba_map[4];
     int step;
@@ -471,8 +471,10 @@ static int query_formats(AVFilterContext *ctx)
         AV_PIX_FMT_RGBA64, AV_PIX_FMT_BGRA64,
         AV_PIX_FMT_NONE
     };
-    ff_set_common_formats(ctx, ff_make_format_list(pix_fmts));
-    return 0;
+    AVFilterFormats *fmts_list = ff_make_format_list(pix_fmts);
+    if (!fmts_list)
+        return AVERROR(ENOMEM);
+    return ff_set_common_formats(ctx, fmts_list);
 }
 
 static int config_input(AVFilterLink *inlink)
@@ -615,11 +617,11 @@ static const AVFilterPad lut3d_inputs[] = {
 };
 
 static const AVFilterPad lut3d_outputs[] = {
-     {
-         .name = "default",
-         .type = AVMEDIA_TYPE_VIDEO,
-     },
-     { NULL }
+    {
+        .name = "default",
+        .type = AVMEDIA_TYPE_VIDEO,
+    },
+    { NULL }
 };
 
 AVFilter ff_vf_lut3d = {
