@@ -80,32 +80,37 @@ static void parse_waveformatex(AVIOContext *pb, AVCodecContext *c)
     }
 }
 
-int ff_get_wav_header(AVFormatContext *s, AVIOContext *pb,
-                      AVCodecContext *codec, int size)
+/* "big_endian" values are needed for RIFX file format */
+int ff_get_wav_header(AVIOContext *pb, AVCodecContext *codec, int size, int big_endian)
 {
     int id;
 
     if (size < 14)
         avpriv_request_sample(codec, "wav header size < 14");
 
-    id                 = avio_rl16(pb);
     codec->codec_type  = AVMEDIA_TYPE_AUDIO;
-    codec->channels    = avio_rl16(pb);
-    codec->sample_rate = avio_rl32(pb);
-    codec->bit_rate    = avio_rl32(pb) * 8;
-    codec->block_align = avio_rl16(pb);
-    if (codec->bit_rate < 0) {
-        av_log(s, AV_LOG_WARNING,
-               "Invalid bit rate: %d\n", codec->bit_rate);
-        if (s->error_recognition & AV_EF_EXPLODE)
-            return AVERROR_INVALIDDATA;
-        else
-            codec->bit_rate = 0;
+    if (!big_endian) {
+        id                 = avio_rl16(pb);
+        codec->channels    = avio_rl16(pb);
+        codec->sample_rate = avio_rl32(pb);
+        codec->bit_rate    = avio_rl32(pb) * 8;
+        codec->block_align = avio_rl16(pb);
+    } else {
+        id                 = avio_rb16(pb);
+        codec->channels    = avio_rb16(pb);
+        codec->sample_rate = avio_rb32(pb);
+        codec->bit_rate    = avio_rb32(pb) * 8;
+        codec->block_align = avio_rb16(pb);
     }
     if (size == 14) {  /* We're dealing with plain vanilla WAVEFORMAT */
         codec->bits_per_coded_sample = 8;
-    } else
-        codec->bits_per_coded_sample = avio_rl16(pb);
+    } else {
+        if (!big_endian) {
+            codec->bits_per_coded_sample = avio_rl16(pb);
+        } else {
+            codec->bits_per_coded_sample = avio_rb16(pb);
+        }
+    }
     if (id == 0xFFFE) {
         codec->codec_tag = 0;
     } else {
@@ -115,6 +120,10 @@ int ff_get_wav_header(AVFormatContext *s, AVIOContext *pb,
     }
     if (size >= 18) {  /* We're obviously dealing with WAVEFORMATEX */
         int cbSize = avio_rl16(pb); /* cbSize */
+        if (big_endian) {
+            avpriv_report_missing_feature(codec, "WAVEFORMATEX support for RIFX files\n");
+            return AVERROR_PATCHWELCOME;
+        }
         size  -= 18;
         cbSize = FFMIN(size, cbSize);
         if (cbSize >= 22 && id == 0xfffe) { /* WAVEFORMATEXTENSIBLE */
@@ -134,7 +143,7 @@ int ff_get_wav_header(AVFormatContext *s, AVIOContext *pb,
             avio_skip(pb, size);
     }
     if (codec->sample_rate <= 0) {
-        av_log(s, AV_LOG_ERROR,
+        av_log(NULL, AV_LOG_ERROR,
                "Invalid sample rate: %d\n", codec->sample_rate);
         return AVERROR_INVALIDDATA;
     }
